<?php

include_once __DIR__ . '/constants.php';
include_once __DIR__ . '/sqlHelper.php';

# from https://stackoverflow.com/questions/21671179/how-to-generate-a-new-guid
# may not be that unique so use it with something other in combination
function _GUID()
{
    if (function_exists('com_create_guid') === true)
    {
        return trim(com_create_guid(), '{}');
    }

    return sprintf('%04X%04X-%04X-%04X-%04X-%04X%04X%04X', mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(16384, 20479), mt_rand(32768, 49151), mt_rand(0, 65535), mt_rand(0, 65535), mt_rand(0, 65535));
}

/**
 * @param $requestDistinctionString MUST BE A VALID FILE NAME
 * @return string
 */
function getUniqueTempUserWorkingSubDir($requestDistinctionString) {

    global $status_code_NoUniqueRequestIdStringProvided;

    if (!isset($requestDistinctionString) || $requestDistinctionString === '') {
        output($status_code_NoUniqueRequestIdStringProvided, 'could not get unique request id string or is empty');
        exit(1);
    }

    //use micro seconds...
    list($usec, $sec) = explode(" ", microtime());
    $timestamp = $sec . '_' . str_replace('.', '-', $usec) . '';

    $timestamp = $timestamp . '-' . $requestDistinctionString . '-' . _GUID();

    return $timestamp;
}

/**
 * @param $conn PDO the db connection
 * @param $config
 * @param $traceString string the trace string from the request
 * @param $processString string this should be the process name or arguments to find it in process managers
 * @param $sourceCode string all user program source code for from request
 */
function addTraceLogToDb($conn, $config, $traceString, $processString, $sourceCode)
{

    global $s_arg_dbTraceTableName;
    global $status_code_DbIssue;
    global $s_arg_insertTraceLogs;


    //0 = never delete AND never insert
    //-x = never delete
    $insertTraceLogs = boolval($config[$s_arg_insertTraceLogs]);

    if ($insertTraceLogs === TRUE) {

        addToDebugTimings("before inserting trace log");

        debug(strlen($sourceCode));

        $tableName = $config[$s_arg_dbTraceTableName];

        $sql = "INSERT INTO `" . $tableName . "` (traceString, processString, sourceCode) VALUES (:traceString, :processString, :sourceCode);";
        $params = array(
            ':traceString' => $traceString,
            ':processString' => $processString,
            ':sourceCode' => $sourceCode
        );

        if (executeSql($conn, $sql, $params) === NULL) {
            //some error
            output($status_code_DbIssue, 'could not add trace log');
            exit(1);
        }

        addToDebugTimings("after inserting trace log");
    }

}

/**
 * @param $conn PDO
 * @param $config
 * @param $s_deleteTraceLogOlderThanMinutes string caller must check if this is a number or not
 */
function removeOldTraceLogs($conn, $config, $s_deleteTraceLogOlderThanMinutes) {

    global $s_arg_dbTraceTableName;
    global $status_code_DbIssue;

    $tableName = $config[$s_arg_dbTraceTableName];
    $deleteTraceLogOlderThanMinutes = intval($s_deleteTraceLogOlderThanMinutes); //gives 0 on error

    //$deleteTraceLogOlderThanMinutes = 0 or -x -> never delete

    //delete old entries... this has a performance impact... maybe move this to a cron job or something??
    if (is_numeric($s_deleteTraceLogOlderThanMinutes)) {

        if ($deleteTraceLogOlderThanMinutes > 0) {

            addToDebugTimings("before deleting old trace logs");

            $sql = "DELETE FROM `" . $tableName . "` WHERE dateTime < (NOW() - INTERVAL :olderThan MINUTE);";
            $params = array(
                ':olderThan' => $deleteTraceLogOlderThanMinutes
            );

            if (executeSql($conn, $sql, $params) === NULL) {
                //some error
                output($status_code_DbIssue, 'could not remove old trace logs');
                exit(1);
            }

            addToDebugTimings("after deleting old trace logs");
        }
        else if ($deleteTraceLogOlderThanMinutes === 0){

            addToDebugTimings("before deleting all old trace logs");

            $sql = "DELETE FROM `" . $tableName . "`;";
            $params = array(
                ':olderThan' => $deleteTraceLogOlderThanMinutes
            );

            if (executeSql($conn, $sql, $params) === NULL) {
                //some error
                output($status_code_DbIssue, 'could not remove all old trace logs');
                exit(1);
            }

            addToDebugTimings("after deleting all old trace logs");
        }

    }

}
